@@ -1,25 +1,15 @@
-# -*- coding: utf8 -*-
-from __future__ import unicode_literals
-
-try:
-    from urllib.parse import urlparse, urljoin
-except ImportError:
-    from urlparse import urlparse, urljoin
-import warnings
-
-try:
-    from collections import OrderedDict
-except ImportError:
-    # python 2.6 compatibility (need to install ordereddict package).
-    from ordereddict import OrderedDict
+from collections import OrderedDict
+from urllib.parse import urlparse
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.utils.translation import ugettext_lazy as _
-from django.utils import six
+from django.templatetags.static import static
+from django.utils.translation import gettext_lazy as _
+
+from .utils import memoized_lazy_function, ListWithLazyItems, ListWithLazyItemsRawIterator
 
 
-DEFAULT_TILES = [(_('OSM'), 'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
+DEFAULT_TILES = [(_('OSM'), '//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                   '© <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors')]
 
 LEAFLET_CONFIG = getattr(settings, 'LEAFLET_CONFIG', {})
@@ -28,11 +18,13 @@ app_settings = dict({
     'TILES': DEFAULT_TILES,
     'OVERLAYS': [],
     'ATTRIBUTION_PREFIX': None,
-    'SPATIAL_EXTENT': None,
+    'LOADEVENT': 'load',
     'DEFAULT_ZOOM': None,
     'MIN_ZOOM': None,
     'MAX_ZOOM': None,
     'DEFAULT_CENTER': None,
+    'DEFAULT_PRECISION': 6,
+    'FORCE_IMAGE_PATH': False,
     'SRID': None,
     'TILES_EXTENT': [],
     'SCALE': 'metric',
@@ -40,19 +32,12 @@ app_settings = dict({
     'RESET_VIEW': True,
     'NO_GLOBALS': True,
     'PLUGINS': OrderedDict(),
+    'SPATIAL_EXTENT': (-180, -90, 180, 90),
 }, **LEAFLET_CONFIG)
 
 
-# Backward-compatibility : defaults TILES with value of TILES_URL
-if 'TILES_URL' in LEAFLET_CONFIG:
-    warnings.warn("TILES_URL is deprecated.", DeprecationWarning)
-    if 'TILES' in LEAFLET_CONFIG:
-        raise ImproperlyConfigured(_("Remove TILES_URL and keep TILES value."))
-    app_settings['TILES'] = [(app_settings['TILES_URL'])]
-
-
 # If TILES is a string, convert to tuple
-if isinstance(app_settings.get('TILES'), six.string_types):
+if isinstance(app_settings.get('TILES'), str):
     app_settings['TILES'] = [(_('Background'), app_settings.get('TILES'), '')]
 
 
@@ -65,24 +50,11 @@ elif SCALE not in ('metric', 'imperial', 'both', None, False):
 
 
 SPATIAL_EXTENT = app_settings.get("SPATIAL_EXTENT")
-if SPATIAL_EXTENT is None:
-    # Deprecate lookup in global Django settings
-    if hasattr(settings, 'SPATIAL_EXTENT'):
-        warnings.warn("SPATIAL_EXTENT is deprecated. Use LEAFLET_CONFIG['SPATIAL_EXTENT'] instead.", DeprecationWarning)
-    SPATIAL_EXTENT = getattr(settings, 'SPATIAL_EXTENT', (-180, -90, 180, 90))
-if SPATIAL_EXTENT is not None:
-    if not isinstance(SPATIAL_EXTENT, (tuple, list)) or len(SPATIAL_EXTENT) != 4:
-        raise ImproperlyConfigured(_("Spatial extent should be a tuple (minx, miny, maxx, maxy)"))
+if SPATIAL_EXTENT is not None and not isinstance(SPATIAL_EXTENT, (tuple, list)) or len(SPATIAL_EXTENT) != 4:
+    raise ImproperlyConfigured(_("Spatial extent should be a tuple (minx, miny, maxx, maxy)"))
 
 
 SRID = app_settings.get("SRID")
-if SRID is None:
-    # Deprecate lookup in global Django settings
-    if hasattr(settings, 'MAP_SRID'):
-        warnings.warn("MAP_SRID is deprecated. Use LEAFLET_CONFIG['SRID'] instead.", DeprecationWarning)
-    if hasattr(settings, 'SRID'):
-        warnings.warn("SRID is deprecated. Use LEAFLET_CONFIG['SRID'] instead.", DeprecationWarning)
-    SRID = getattr(settings, 'MAP_SRID', getattr(settings, 'SRID', 3857))
 if SRID == 3857:  # Leaflet's default, do not setup custom projection machinery
     SRID = None
 
@@ -100,10 +72,15 @@ if DEFAULT_CENTER is not None and not (isinstance(DEFAULT_CENTER, (list, tuple))
 
 
 DEFAULT_ZOOM = app_settings['DEFAULT_ZOOM']
-if DEFAULT_ZOOM is not None and not (isinstance(DEFAULT_ZOOM, six.integer_types) and (1 <= DEFAULT_ZOOM <= 24)):
+if DEFAULT_ZOOM is not None and not (isinstance(DEFAULT_ZOOM, int) and (1 <= DEFAULT_ZOOM <= 24)):
     raise ImproperlyConfigured("LEAFLET_CONFIG['DEFAULT_ZOOM'] must be an int between 1 and 24.")
 
 
+DEFAULT_PRECISION = app_settings['DEFAULT_PRECISION']
+if DEFAULT_PRECISION is not None and not (isinstance(DEFAULT_PRECISION, int) and (4 <= DEFAULT_PRECISION <= 12)):
+    raise ImproperlyConfigured("LEAFLET_CONFIG['DEFAULT_PRECISION'] must be an int between 4 and 12.")
+
+
 PLUGINS = app_settings['PLUGINS']
 if not (isinstance(PLUGINS, dict) and all([isinstance(el, dict) for el in PLUGINS.values()])):
     error_msg = """LEAFLET_CONFIG['PLUGINS'] must be dict of dicts in the format:
@@ -120,7 +97,7 @@ _forms_js = ['leaflet/draw/leaflet.draw.js',
              'leaflet/leaflet.extras.js',
              'leaflet/leaflet.forms.js']
 if SRID:
-    _forms_js += ['leaflet/proj4js.js',
+    _forms_js += ['leaflet/proj4.js',
                   'leaflet/proj4leaflet.js',
                   'proj4js/%s.js' % SRID]
 
@@ -165,33 +142,44 @@ def _normalize_plugins_config():
         for resource_type in RESOURCE_TYPE_KEYS:
             # normalize the resource URLs
             urls = plugin_dict.get(resource_type, None)
-            if isinstance(urls, (six.binary_type, six.string_types)):
+            if isinstance(urls, str):
                 urls = [urls]
             elif isinstance(urls, tuple):  # force to list
                 urls = list(urls)
             elif isinstance(urls, list):  # already a list
                 pass
+            elif isinstance(urls, ListWithLazyItems):
+                # prevent evaluating Promises too early
+                urls = ListWithLazyItemsRawIterator(urls)
             else:  # css/js has not been specified or the wrong type
                 urls = []
 
             # normalize the URLs - see the docstring for details
             for i, url in enumerate(urls):
+                if ListWithLazyItems.is_lazy_item(url):
+                    # If it is a Promise, then we have already
+                    # seen this url and have lazily applied the `static` call
+                    # to it, so we can safely skip the check below.
+                    continue
                 url_parts = urlparse(url)
                 if url_parts.scheme or url_parts.path.startswith('/'):
                     # absolute URL or a URL starting at root
                     pass
                 else:
-                    urls[i] = urljoin(settings.STATIC_URL, url)
+                    # pass relative URL through django.contrib.staticfiles
+                    urls[i] = memoized_lazy_function(static, url)  # lazy variant of `static(url)`
 
+            urls = ListWithLazyItems(urls)
             plugin_dict[resource_type] = urls
 
             # Append it to the DEFAULT pseudo-plugin if auto-include
             if plugin_dict.get('auto-include', False):
-                PLUGINS[PLUGINS_DEFAULT].setdefault(resource_type, []).extend(urls)
+                PLUGINS[PLUGINS_DEFAULT].setdefault(resource_type, ListWithLazyItems()).extend(urls)
 
             # also append it to the ALL pseudo-plugin;
-            PLUGINS[PLUGIN_ALL].setdefault(resource_type, []).extend(urls)
+            PLUGINS[PLUGIN_ALL].setdefault(resource_type, ListWithLazyItems()).extend(urls)
 
     PLUGINS['__is_normalized__'] = True
 
-_normalize_plugins_config()
+
+default_app_config = 'leaflet.apps.LeafletConfig'
